// Need to ensure these variables are not optimized out
// since they persist across interrupts
volatile uint8_t curr_state3 = 0;
volatile uint8_t curr_state1 = 0;
volatile uint8_t curr_state2 = 0;
volatile uint8_t curr_state4 = 0;
volatile uint8_t curr_state5 = 0;
volatile uint8_t curr_state6 = 0;

// Some motors are different than others and respond differently
// to the pulsed routine
const uint8_t currentThreshold = 60;
const uint8_t currentThreshold6 = 60;
const uint8_t currentThreshold4 = 70;
const uint8_t currentThreshold2 = 50;

 #define M6CS A0 // Motor 6 Current Sensor
 #define M1CS A1 // Motor 1 Current Sensor
 #define M6EN 22 // Motor 6 Enable
 #define M1EN 25 // Motor 1 Enable
 #define M6INA 23 // Motor 6 Input A
 #define M6INB 24 // Motor 6 Input B
 #define M1INA 26 // Motor 1 Input A
 #define M1INB 27 // Motor 1 Input B
 #define M6PWM 2 // Motor 6 PWM Signal
 #define M1PWM 3 // Motor 1 PWM Signal
 
 #define M4CS A2 // Motor 4 Current Sensor
 #define M5CS A3 // Motor 5 Current Sensor
 #define M4EN 28 // Motor 4 Enable
 #define M5EN 31 // Motor 5 Enable
 #define M4INA 29 // Motor 4 Input A
 #define M4INB 30 // Motor 4 Input B
 #define M5INA 32 // Motor 5 Input A
 #define M5INB 33 // Motor 5 Input B
 #define M4PWM 4 // Motor 4 PWM Signal
 #define M5PWM 5 // Motor 5 PWM Signal

 #define M3CS A4 // Motor 3 Current Sensor
 #define M2CS A5 // Motor 2 Current Sensor
 #define M3EN 34 // Motor 3 Enable
 #define M2EN 37 // Motor 2 Enable
 #define M3INA 35 // Motor 3 Input A
 #define M3INB 36 // Motor 3 Input B
 #define M2INA 38 // Motor 2 Input A
 #define M2INB 39 // Motor 2 Input B
 #define M3PWM 6 // Motor 3 PWM Signal
 #define M2PWM 7 // Motor 2 PWM Signal


 #define TRIG1 40 // Ultrasonic 1 Trigger
 #define ECHO1 41 // Ultrasonic 1 Echo

 #define TRIG2 42 // Ultrasonic 2 Trigger
 #define ECHO2 43 // Ultrasonic 2 Echo

 #define LED   50 // LED for bluetooth verification
 
void setup(){
Serial.begin(9600);

// Need to configure pins with port mapping
pinMode(M1CS, INPUT);
pinMode(M1EN, OUTPUT);
pinMode(M1INA, OUTPUT);
pinMode(M1INB, OUTPUT);
pinMode(M1PWM, OUTPUT);
pinMode(M6CS, INPUT);
pinMode(M6EN, OUTPUT);
pinMode(M6INA, OUTPUT);
pinMode(M6INB, OUTPUT);
pinMode(M6PWM, OUTPUT);
pinMode(M2CS, INPUT);
pinMode(M2EN, OUTPUT);
pinMode(M2INA, OUTPUT);
pinMode(M2INB, OUTPUT);
pinMode(M2PWM, OUTPUT);
pinMode(M3CS, INPUT);
pinMode(M3EN, OUTPUT);
pinMode(M3INA, OUTPUT);
pinMode(M3INB, OUTPUT);
pinMode(M3PWM, OUTPUT);
pinMode(M4CS, INPUT);
pinMode(M4EN, OUTPUT);
pinMode(M4INA, OUTPUT);
pinMode(M4INB, OUTPUT);
pinMode(M4PWM, OUTPUT);
pinMode(M5CS, INPUT);
pinMode(M5EN, OUTPUT);
pinMode(M5INA, OUTPUT);
pinMode(M5INB, OUTPUT);
pinMode(M5PWM, OUTPUT);

cli();//stop interrupts


//set timer5 interrupt at 1 Hz
  TCCR5A = 0;// set entire TCCR2A register to 0
  TCCR5B = 0;// same for TCCR2B
  TCNT5  = 0;//initialize counter value to 0
  // set compare match register for 1 Hz increments
  OCR5A = 15624;// = (16*10^6) / (1024) - 1 (must be <65536)
  // turn on CTC mode with WGM52
  // Set CS52 and CS50 bit for 1024 prescaler
  TCCR5B |= (1 << CS52)|(1 << WGM52)|(1<<CS50);   
  // enable timer compare interrupt
  TIMSK5 |= (1 << OCIE5A);


motorsforward(200,200,200,200,200,200);
sei();//allow interrupts
delay(5000);
}//end setup

  
ISR(TIMER5_COMPA_vect){ // Interrupts every second
  // This routine takes about 360 mS and might upset the value from 
  // 6 wheel differential. Due to this, this ISR is only performed every second.
  // Need to average values of currentReading to get a better estimate of current
  // passing through motors. 
  // No-Load FSM for Motor 6
  // State 0.6
  uint8_t next_state6;
  if(curr_state6 == 0){
  // Check motors to see if they are under no load
  // Can be simplified by creating an averaging filter but this will do
  uint16_t currentReading6_1 = analogRead(M6CS);
  uint16_t currentReading6_2 = analogRead(M6CS);
  uint16_t currentReading6_3 = analogRead(M6CS);
  uint16_t currentReadingave6 = (currentReading6_1 +currentReading6_2 +currentReading6_3)/3;
    if (currentReadingave6 < currentThreshold){ // Go to state 1
      digitalWrite(M6EN,LOW);
      next_state6 = 1;
    }
    else{next_state6 = 0;} // Remain in state 0
    }
  // State 1.6
  if(curr_state6 = 1){
    // Apply power to motors and check if they are under load
    digitalWrite(M6EN, HIGH);
    sei();
    delay(60);
    cli();
    uint16_t currentReading6_1 = analogRead(M6CS);
    uint16_t currentReading6_2 = analogRead(M6CS);
    uint16_t currentReading6_3 = analogRead(M6CS);
    uint16_t currentReadingave6 = (currentReading6_1 +currentReading6_2 +currentReading6_3)/3;
    digitalWrite(M6EN, LOW);
        if (currentReadingave6 < currentThreshold){ // Remain in State 1
      next_state6 = 1;
    }
    else{next_state6 = 0; digitalWrite(M6EN,HIGH);} // Otherwise go to state 0
    }
  curr_state6 = next_state6;
  
// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
  // No-Load FSM for Motor 1
  // State 0.1
  uint8_t next_state1;
  if(curr_state1 == 0){
  // Check motors to see if they are under no load
  uint16_t currentReading1_1 = analogRead(M1CS);
  uint16_t currentReading1_2 = analogRead(M1CS);
  uint16_t currentReading1_3 = analogRead(M1CS);
  uint16_t currentReadingave1 = (currentReading1_1 +currentReading1_2 +currentReading1_3)/3;
    if (currentReadingave1 < currentThreshold){ // Go to state 1
      digitalWrite(M1EN,LOW);
      next_state1 = 1;
    }
    else{next_state1 = 0;} // Remain in state 0
    }
  // State 1.1
  if(curr_state1 = 1){
    // Apply power to motors and check if they are under load
    digitalWrite(M1EN, HIGH);
    sei();
    delay(60);
    cli();
    uint16_t currentReading1_1 = analogRead(M1CS);
    uint16_t currentReading1_2 = analogRead(M1CS);
    uint16_t currentReading1_3 = analogRead(M1CS);
    uint16_t currentReadingave1 = (currentReading1_1 +currentReading1_2 +currentReading1_3)/3;
    digitalWrite(M1EN, LOW);
        if (currentReadingave1 < currentThreshold){ // Remain in State 1
      next_state1 = 1;
    }
    else{next_state1 = 0; digitalWrite(M1EN,HIGH);} // Otherwise go to state 0
    }
  curr_state1 = next_state1;

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // No-Load FSM for Motor 2
  // State 0.2
  uint8_t next_state2;
  if(curr_state2 == 0){
  // Check motors to see if they are under no load
  uint16_t currentReading2_1 = analogRead(M2CS);
  uint16_t currentReading2_2 = analogRead(M2CS);
  uint16_t currentReading2_3 = analogRead(M2CS);
  uint16_t currentReadingave2 = (currentReading2_1 +currentReading2_2 +currentReading2_3)/3;
    if (currentReadingave2 < currentThreshold){ // Go to state 1
      digitalWrite(M2EN,LOW);
      next_state2 = 1;
    }
    else{next_state2 = 0;} // Remain in state 0
    }
  // State 2.1
  if(curr_state2 == 1){
    // Apply power to motors and check if they are under load
    digitalWrite(M2EN, HIGH);
    sei();
    delay(60);
    cli();
    uint16_t currentReading2_1 = analogRead(M2CS);
    uint16_t currentReading2_2 = analogRead(M2CS);
    uint16_t currentReading2_3 = analogRead(M2CS);
    uint16_t currentReadingave2 = (currentReading2_1 +currentReading2_2 +currentReading2_3)/3;
    digitalWrite(M2EN, LOW);
        if (currentReadingave2 < currentThreshold){ // Remain in State 1
      next_state2 = 1;
    }
    else{next_state2 = 0; digitalWrite(M2EN,HIGH);} // Otherwise go to state 0
    }
  curr_state2 = next_state2;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // No-Load FSM for Motor 3
  // State 0.3
  uint8_t next_state3;
  if(curr_state3 == 0){
  // Check motors to see if they are under no load
  uint16_t currentReading3_1 = analogRead(M3CS);
  uint16_t currentReading3_2 = analogRead(M3CS);
  uint16_t currentReading3_3 = analogRead(M3CS);
  uint16_t currentReadingave3 = (currentReading3_1 +currentReading3_2 +currentReading3_3)/3;
    if (currentReadingave3 < currentThreshold){ // Go to state 1
      digitalWrite(M3EN,LOW);
      next_state3 = 1;
    }
    else{next_state3 = 0;} // Remain in state 0
    }
  // State 1.3
  if(curr_state3 == 1){
    // Apply power to motors and check if they are under load
    digitalWrite(M3EN, HIGH);
    sei();
    delay(60);
    cli();
    uint16_t currentReading3_1 = analogRead(M3CS);
    uint16_t currentReading3_2 = analogRead(M3CS);
    uint16_t currentReading3_3 = analogRead(M3CS);
    uint16_t currentReadingave3 = (currentReading3_1 +currentReading3_2 +currentReading3_3)/3;
    digitalWrite(M3EN, LOW);
        if (currentReadingave3 < currentThreshold){ // Remain in State 1
      next_state3 = 1;
    }
    else{next_state3 = 0; digitalWrite(M3EN,HIGH);} // Otherwise go to state 0
    }
  curr_state3 = next_state3;

  uint8_t next_state4;
  if(curr_state4 == 0){
  // Check motors to see if they are under no load
  uint16_t currentReading4_1 = analogRead(M4CS);
  uint16_t currentReading4_2 = analogRead(M4CS);
  uint16_t currentReading4_3 = analogRead(M4CS);
  uint16_t currentReadingave4 = (currentReading4_1 +currentReading4_2 +currentReading4_3)/3;
    if (currentReadingave4 < currentThreshold4){ // Go to state 1
      digitalWrite(M4EN,LOW);
      next_state4 = 1;
    }
    else{next_state4 = 0;} // Remain in state 0
    }
  // State 4.1
  if(curr_state4 = 1){
    // Apply power to motors and check if they are under load
    digitalWrite(M4EN, HIGH);
    sei();
    delay(60);
    cli();
    uint16_t currentReading4_1 = analogRead(M4CS);
    uint16_t currentReading4_2 = analogRead(M4CS);
    uint16_t currentReading4_3 = analogRead(M4CS);
    uint16_t currentReadingave4 = (currentReading4_1 +currentReading4_2 +currentReading4_3)/3;
    digitalWrite(M4EN, LOW);
        if (currentReadingave4 < currentThreshold4){ // Remain in State 1
      next_state4 = 1;
    }
    else{next_state4 = 0; digitalWrite(M4EN,HIGH);} // Otherwise go to state 0
    }
  curr_state4 = next_state4;

  uint8_t next_state5;
  if(curr_state5 == 0){
  // Check motors to see if they are under no load
  uint16_t currentReading5_1 = analogRead(M5CS);
  uint16_t currentReading5_2 = analogRead(M5CS);
  uint16_t currentReading5_3 = analogRead(M5CS);
  uint16_t currentReadingave5 = (currentReading5_1 +currentReading5_2 +currentReading5_3)/3;
    if (currentReadingave5 < currentThreshold){ // Go to state 1
      digitalWrite(M5EN,LOW);
      next_state5 = 1;
    }
    else{next_state5 = 0;} // Remain in state 0
    }
  // State 5.1
  if(curr_state5 = 1){
    // Apply power to motors and check if they are under load
    digitalWrite(M5EN, HIGH);
    sei();
    delay(60);
    cli();
    uint16_t currentReading5_1 = analogRead(M5CS);
    uint16_t currentReading5_2 = analogRead(M5CS);
    uint16_t currentReading5_3 = analogRead(M5CS);
    uint16_t currentReadingave5 = (currentReading5_1 +currentReading5_2 +currentReading5_3)/3;
    digitalWrite(M5EN, LOW);
        if (currentReadingave5 < currentThreshold){ // Remain in State 1
      next_state5 = 1;
    }
    else{next_state5 = 0; digitalWrite(M5EN,HIGH);} // Otherwise go to state 0
    }
  curr_state5 = next_state5;


 
}


void loop(){
}

