
#include <EEPROM.h>
#include <Wire.h>
#include <Configure.h>
#include <FuelGauge.h>
#include <Motor.h>
#include <Packet.h>
#include <Robot3DoTBoard.h>
#include <TB6612FNG.h>
#include <TelecomClass.h>
#include <Watchdog.h>
#include <math.h>

Robot3DoTBoard Robot3DoT;       // instantiated as Robot3DoT at end of class header
/*
 * This code will do the following:
 * 
 * When uploaded to the Arduino Mega, the pathfinder will begin communication with the Arxterra robot application.
 * It will wait until the user decides to begin the demonstration by selecting the demo command on the application.
 * This demo will involve the robot going forward, then going in reverse and then performing two spins. 
 * Once it has completed the demonstration it will stop and wait to either be powered off or repeat the demo.
 */

/*
 * Author: Jordan Smallwood
 * SID#: 015277455
 * EE 400D
 * Professor Gary Hill
 * California State University, Long Beach
 */
/* 
 *  The left 3 motors as viewed from the top of the pathfinder are defined as motors 4,
 *  5 & 6. The right three motors are 1, 2 & 3. 
 */

 
 long waypointLat4;
 long waypointLat3;
 long waypointLat2;
 long waypointLat1;
 unsigned long waypointLat;
 long waypointLon4;
 long waypointLon3;
 long waypointLon2;
 long waypointLon1;
 unsigned long waypointLon;

 long currentLat4;
 long currentLat3;
 long currentLat2;
 long currentLat1;
 unsigned long currentLat;
 long currentLon4;
 long currentLon3;
 long currentLon2;
 long currentLon1;
 unsigned long currentLon;
 
 const float dLat = 0.01111;
 const float dLong = 0.00927;
 float heading;
 float dir;
 float turn;
 float turndir;
 float dist;
 
const uint8_t CMD_LIST_SIZE = 3; // total number of custom commands
void waypoint_coordHandler(uint8_t cmd, uint8_t param[], uint8_t n);
void current_coordHandler(uint8_t cmd, uint8_t param[], uint8_t n);
void headingHandler(uint8_t cmd, uint8_t param[], uint8_t n);


Robot3DoTBoard::cmdFunc_t onCommand[CMD_LIST_SIZE] = {{WAYPOINT_COORD,waypoint_coordHandler},{CURRENT_COORD,current_coordHandler},{HEADING, headingHandler}};

void setup() {
  // put your setup code here, to run once:
Serial.begin(9600);
Serial1.begin(9600);
Robot3DoT.begin();
Robot3DoT.setOnCommand(onCommand, CMD_LIST_SIZE);

}

void loop() {
  Robot3DoT.loop();
  calcDesiredHeadingandDistance();
  Serial.print("Distance: ");
  Serial.println(dist);
  Serial.print("Direction: ");
  Serial.println(dir);
}


// This functon will handle the current coordinate, I.E store the coordinate for use in calculations
 void current_coordHandler(uint8_t cmd, uint8_t param[], uint8_t n){
  currentLat4 = param[0];
  currentLat3 = param[1];
  currentLat2 = param[2];
  currentLat1 = param[3];
  currentLat = (currentLat4 << 24 )| (currentLat3 << 16)| (currentLat2 << 8)| (currentLat1);
  currentLon4 = param[4];
  currentLon3 = param[5];
  currentLon2 = param[6];
  currentLon1 = param[7];
  currentLon = (currentLon4 << 24) |( currentLon3 << 16)|( currentLon2 << 8)|( currentLon1);
  currentLon = ~currentLon;
  currentLon = (currentLon + 1) * -1;
}

// This functon will handle the waypoint coordinate, I.E store the coordinate for use in calculations
 void waypoint_coordHandler(uint8_t cmd, uint8_t param[], uint8_t n){
  waypointLat4 = param[0];
  waypointLat3 = param[1];
  waypointLat2 = param[2];
  waypointLat1 = param[3];
  waypointLat = (waypointLat4 << 24) | (waypointLat3 << 16)| (waypointLat2 << 8)| (waypointLat1);
  waypointLon4 = param[4];
  waypointLon3 = param[5];
  waypointLon2 = param[6];
  waypointLon1 = param[7];
  waypointLon = (waypointLon4 << 24) |( waypointLon3 << 16)|( waypointLon2 << 8)|( waypointLon1);
  waypointLon = ~waypointLon;
  waypointLon = (waypointLon + 1) * -1;
}

void headingHandler(uint8_t cmd, uint8_t param[], uint8_t n){
      // received automatically  
      // move heading data bytes into long
      long currentHeading = ((long)param[3]) << 8;    // heading byte 1
      currentHeading |= (long)param[4];          // heading byte 0
    
      // convert to float and send to global var
      // represented in degrees
      heading = (float)currentHeading;
}
